package com.persagy.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.cache.AlarmLastTimeCache;
import com.persagy.cache.CreatedAlarmIdsCache;
import com.persagy.cache.CurrentDataCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.*;
import com.persagy.job.ExpireAlarmQueue;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.service.AlarmHandleService;
import com.persagy.service.AlarmQuartzService;
import com.persagy.utils.DateUtils;
import com.persagy.utils.StringUtil;
import com.persagy.utils.ValidateUtils;
import com.persagy.utils.condition.ConditionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.JobDataMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @description:报警处理实现类：包含报警产生和报警恢复
 * @author:LuoGuanhyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/15 001511:14
 * @version:V1.0
 **/
@Service
@Slf4j
public class AlarmHandleServiceImpl implements AlarmHandleService {
    @Autowired
    AlarmInfoCache alarmInfoCache;
    @Autowired
    CurrentDataCache currentDataCache;
    @Autowired
    GroupNettyClient groupNettyClient;
    @Autowired
    AlarmQuartzService alarmQuartzService;
    @Autowired
    AlarmRecordRepository alarmRecordRepository;
    @Autowired
    CreatedAlarmIdsCache createdAlarmIdsCache;
    @Autowired
    AlarmLastTimeCache alarmLastTimeCache;
    @Autowired
    UserDefinePeriod userDefinePeriod;

    /**
     * 集团编码
     */
    @Value("${group.code}")
    public String groupCode;
    /**
     * 系统标识，用于创建人
     */
    @Value("${system.id}")
    public String systemId;

    /**
     * @description: 处理iot采集值消息
     * @param: msg iot采集值消息
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/1 3:31 下午
     * @version: V1.0
     */
    @Override
    public void handleIotMsg(String msg) throws Exception {
        // 判断当前时间是否在定义的时间段内，如果不在则不处理
        if (!userDefinePeriod.inUserDefinePeriod(new Date())) {
            return;
        }

        log.info("接收到采集值：[{}]", msg);
        // 校验采集值
        boolean validateIotMsgResult = validateIotMsg(msg);
        if (!validateIotMsgResult) {
            log.error("采集值校验未通过");
            return;
        }

        /* 将iot消息拆分成一条条iot数据，然后处理 */
        String[] split = msg.split(";");
        // 每一条iot数据应该包含4个分号
        int groupCount = 4;
        int dataCounts = split.length / groupCount;
        for (int i = 0; i < dataCounts; i++) {
            int startIndex = i * groupCount;
            String dataTime = split[startIndex];
            String meterId = split[startIndex + 1];
            String funcId = split[startIndex + 2];
            String value = split[startIndex + 3];
            // 使用intern（）方法，确保上锁的是同一个String对象
            synchronized (meterId.intern()) {
                // 处理iot数据
                handleIotData(dataTime, meterId, funcId, value);
            }
        }
    }

//    public static void main(String[] args) throws Exception {
//        String msg = "20210331094500;Eq4403050002cfb2cd6c7475465397e8738ee9ada861;Pclose,EnergyData,CsLJCWDetaTemp;0.1,0.81,1.2;";
//        AlarmHandleServiceImpl alarmHandleService = new AlarmHandleServiceImpl();
//        alarmHandleService.handleIotMsg(msg);
//    }

    /**
     * @description: 处理iot数据, iot数据可能包含一个设备多个信息点的采集值
     * @param: dataTime 采集时间
     * @param: meterId 设备id
     * @param: funcIdStr 信息点id
     * @param: valueStr 采集值
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/1 3:44 下午
     * @version: V1.0
     */
    private void handleIotData(String dataTime, String meterId, String funcIdStr, String valueStr) throws Exception {
        /* 如果iot数据为一组数据，先更新缓存中这组iot数据值，然后将这一组iot数据拆分为一条条iot数据处理 */
        String funcIdSeparator = ",";
        String[] funcIdArray = funcIdStr.split(funcIdSeparator);
        String[] values = valueStr.split(funcIdSeparator);
        // 更新缓存中的设备信息点当前采集值
        for (int i = 0; i < funcIdArray.length; i++) {
            // 如果有对应的报警定义
            if (alarmInfoCache.hasKey(meterId, funcIdArray[i])) {
                currentDataCache.putCurrentData(meterId, funcIdArray[i], Double.parseDouble(values[i]));
            } else {
                log.info("未获取到对应的报警定义, meterId:{}, funcId:{}", meterId, funcIdArray[i]);
            }
        }

//        // 将一组数据拆分为单条处理
//        for (String funcId : funcIdArray) {
//            if (alarmInfoCache.hasKey(meterId, funcId)) {
//                handleSingleIotData(dataTime, meterId, funcId);
//            } else {
//                log.info("未获取到对应的报警定义, meterId:{}, funcId:{}", meterId, funcId);
//            }
//        }
        // 因为所有功能点的数据都被缓存了，所以这里只处理一个功能点即可
        String funcId = funcIdArray[0];
        if (alarmInfoCache.hasKey(meterId, funcId)) {
            handleSingleIotData(dataTime, meterId, funcId);
        } else {
            log.info("未获取到对应的报警定义, meterId:{}, funcId:{}", meterId, funcId);
        }

    }

    /**
     * @description: 处理单条iot数据
     * @param: dataTime 采集时间
     * @param: meterId 设备id
     * @param: funcId 信息点id
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/1 4:01 下午
     * @version: V1.0
     */
    private void handleSingleIotData(String dataTime, String meterId, String funcId) throws Exception {
        // 获取和采集值相关的报警定义
        List<AlarmDefine> alarmDefines = alarmInfoCache.getAlarmDefinitionsByMeterFuncId(meterId, funcId);
        for (AlarmDefine alarmDefine : alarmDefines) {
            Condition condition = alarmDefine.getCondition();
            String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
            log.info("defineId: [{}]", defineId);

            // 判断报警定义所包含的信息点在缓存中是否都有值
            List<JSONObject> infoCodes = condition.getInfoCodes();
            boolean infoCodesFullFill = infoCodes.stream().allMatch(
                    p -> currentDataCache.hasKey(p.getString("meterId"), p.getString("funcId"))
            );

            //报警定义的所有信息点都有采集数值，具备判断条件
            if (infoCodesFullFill) {
                // 解析触发和恢复条件
                String trigger = condition.getTrigger();
                Expression triggerExp = AviatorEvaluator.compile(trigger, true);
                String end = condition.getEnd();
                Expression endExp = AviatorEvaluator.compile(end, true);

                // 拼装报警条件参数
                HashMap<String, Object> paramMap = new HashMap<>();
                for (JSONObject code : infoCodes) {
                    paramMap.put(
                            code.getString("infoCode"),
                            currentDataCache.getCurrentData(code.getString("meterId"), code.getString("funcId")));
                }

                // 执行报警条件公式，得到触发和恢复结果
                Boolean triggerResult = null;
                Boolean endResult = null;
                try {
                    triggerResult = (Boolean) triggerExp.execute(paramMap);
                    endResult = (Boolean) endExp.execute(paramMap);
                } catch (Exception e) {
                    log.error("报警条件解析异常");
                    log.error("触发条件：");
                    log.error(trigger);
                    log.error("恢复条件：");
                    log.error(end);
                    log.error("参数：");
                    log.error(paramMap.toString());
                    return;
                }
                log.info("triggerResult:[{}],endResult:[{}]", triggerResult, endResult);
                if (triggerResult && endResult) {
                    log.error("报警触发条件和报警恢复条件同时满足，请检查，报警定义详情【{}】", alarmDefine);
                }
                synchronized (defineId.intern()) {
                    //报警产生值满足（这里的满足不考虑报警持续时间）
                    if (triggerResult) {
                        log.info("满足报警条件");
                        //报警的时候不考虑报警恢复，因为同时报警和报警恢复是不应该出现的
                        handleAlarmTriggerData(alarmDefine, dataTime, paramMap);
                    } else {
                        log.info("不满足报警条件");
                        //当前数据正常
                        handlerNormalData(alarmDefine, dataTime, endResult, paramMap);
                    }
                }
            } else {
                log.warn("部分信息点没有数值");
            }
        }
    }

    /**
     * @description: 校验iot消息是否合法
     * @param: msg iot数据消息
     * @return: boolean 校验结果
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/1 2:26 下午
     * @version: V1.0
     */
    private boolean validateIotMsg(String msg) {
        // iot数据使用英文分号分隔
        String separator = ";";
        // 每一条iot数据应该包含4个分号
        int groupCount = 4;
        String[] split = msg.split(separator);
        if (split.length % groupCount != 0) {
            log.error("采集值有误！每一条iot数据应该包含4个英文分号");
            return false;
        }
        // 数据条数
        int dataCounts = (split.length) / groupCount;
        for (int i = 0; i < dataCounts; i++) {
            int startIndex = i * groupCount;
            String dataTime = split[startIndex];
            String meterId = split[startIndex + 1];
            String funcId = split[startIndex + 2];
            String value = split[startIndex + 3];

            boolean iotDataValidateResult = validateIotData(dataTime, meterId, funcId, value);
            if (!iotDataValidateResult) {
                return false;
            }
        }
        return true;
    }

    /**
     * @description: 校验iot数据是否合法
     * @param: dataTime 采集时间
     * @param: meterId 设备id
     * @param: funcId 信息点id
     * @param: value 采集值
     * @return: boolean 校验结果
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/1 3:22 下午
     * @version: V1.0
     */
    private boolean validateIotData(String dataTime, String meterId, String funcId, String value) {
        String separator = ",";
        // 校验时间格式
        boolean dateTimeValid = ValidateUtils.validateDateTime(dataTime);
        if (!dateTimeValid) {
            log.error("采集值时间有误，[{}]", dataTime);
            return false;
        }

        // 校验信息点和值是否匹配
        int funcIdsLength = funcId.split(separator).length;
        int valuesLength = value.split(separator).length;
        if (funcIdsLength != valuesLength) {
            log.error("信息点个数与值的个数不匹配");
            return false;
        }

        // 校验值是否为合法的值
        if (!value.contains(separator)) {
            try {
                Double.parseDouble(value);
            } catch (NumberFormatException e) {
                log.error("采集值不能被转换为数字，[{}]", value);
                return false;
            }
        } else {
            String[] values = value.split(separator);
            for (String tmp : values) {
                try {
                    Double.parseDouble(tmp);
                } catch (NumberFormatException e) {
                    log.error("采集值不能被转换为数字，[{}]", value);
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * @description: 根据报警定义唯一标识从缓存或数据库中获取AlarmDefineState
     * @param: defineId 报警定义唯一标识
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 上午11:05
     * @version: V1.0
     */
    private AlarmDefineState getAlarmDefineStateFromCacheOrDb(String defineId) {
        AlarmDefineState alarmDefineState = alarmInfoCache.getAlarmDefineState(defineId);
        if (Objects.isNull(alarmDefineState)) {
            //默认正常报警状态
            alarmDefineState = new AlarmDefineState(defineId);
            Optional<ZktAlarmRecordDO> recordOptional = alarmRecordRepository.findById(defineId);
            if (recordOptional.isPresent()) {
                ZktAlarmRecordDO alarmRecordDO = recordOptional.get();
                //数据库报警状态：1-未处理
                if ("1".equals(alarmRecordDO.getState())) {
                    alarmDefineState.setState(AlarmDefineState.State.NOT_DEAL.getType());
                    alarmDefineState.setAlarmStartTime(alarmRecordDO.getAlarmTime());
                }
            }
        }
        log.info("alarmDefineState: [{}], （ 0-正常 1-报警）", alarmDefineState.getState());
        return alarmDefineState;
    }

    /**
     * @param alarmDefine :报警定义
     * @param dataTime    :IOT采集时间
     * @param endResult   :报警恢复监测结果
     * @param paramMap    报警恢复触发值
     * @description: 当前数据正常判断逻辑
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @return: void
     * @since: 2020/10/21 16:46
     * @version: V1.0
     */
    private void handlerNormalData(
            AlarmDefine alarmDefine, String dataTime,
            Boolean endResult, HashMap<String, Object> paramMap) throws InterruptedException {
        String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
        AlarmDefineState alarmDefineState = getAlarmDefineStateFromCacheOrDb(defineId);

        Condition condition = alarmDefine.getCondition();
        //当前数据正常，报警状态为正常：清空之前的报警计时，重置回默认状态
        if (AlarmDefineState.State.NORMAL.getType().equals(alarmDefineState.getState())) {
            alarmDefineState = new AlarmDefineState(defineId);
        } else if (AlarmDefineState.State.NOT_DEAL.getType().equals(alarmDefineState.getState())) {
            // 报警恢复条件不满足，不处理
            if (!endResult) {
                return;
            }
            /* 判断是否满足报警恢复持续时间 */
            String alarmSuspendStartTime = alarmDefineState.getAlarmSuspendStartTime();
            if (StringUtil.isEmpty(alarmSuspendStartTime)) {
                alarmSuspendStartTime = dataTime;
                //设置开始恢复时间
                alarmDefineState.setAlarmSuspendStartTime(alarmSuspendStartTime);
            }
            // 计算报警恢复持续时间
            long alarmSuspendLastTime = DateUtils.betweenTwoTimeSecond(alarmSuspendStartTime, dataTime);
            // 设定的报警恢复持续时间
            int uphold = condition.getEndUphold();
            if (alarmSuspendLastTime < uphold) {
                return;
            }

            /* 超过报警恢复设置的持续时间 */
            log.info("报警恢复持续时间大于设定时间：[{}]>[{}]", alarmSuspendLastTime, uphold);

            ZktAlarmRecordDO alarmRecordDO = getZktAlarmRecordDOWhenAlarmSuspend(defineId, dataTime, paramMap);
            String alarmId = alarmRecordDO.getAlarmId();
            // 报警恢复时，alarmLastTimeCache中对应的报警标记为可删除
            alarmLastTimeCache.setDeleteAvailable(defineId);
            // 如果云端已经完成报警记录的创建，直接发送更新报警状态消息，并删除数据库中的报警信息
            if (createdAlarmIdsCache.contains(alarmId)) {
                deleteZktAlarmRecordWhenAlarmSuspend(defineId);
                log.info("发送报警恢复消息，报警id: [{}]", alarmId);
                // 发送报警恢复消息
                sendAlarmRecoverMessage(alarmDefine.getProjectId(), dataTime, paramMap, alarmId);
                // 报警恢复后，从缓存中移除报警id
                createdAlarmIdsCache.remove(alarmId);
            } else {
                log.info("已创建的报警id中不包含[{}], 3分钟后重试发送报警恢复消息", alarmId);
                // 创建恢复定时任务
                createAlarmRecoverTimingJob(dataTime, paramMap, alarmRecordDO);
            }
            // 取消过期任务
            cancelRelatedExpireJob(alarmId);
            //报警恢复，报警状态重置回默认
            alarmDefineState.reset();
        }
        // 更新报警定义缓存
        alarmInfoCache.setAlarmState(defineId, alarmDefineState);
    }

    /**
     * @description: 创建报警恢复定时任务
     * @param: recoverTime 恢复时间
     * @param: paramMap 采集值参数
     * @param: alarmRecordDO 报警定义对象
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/7 10:10 上午
     * @version: V1.0
     */
    private void createAlarmRecoverTimingJob(
            String recoverTime, HashMap<String, Object> paramMap, ZktAlarmRecordDO alarmRecordDO)
            throws InterruptedException {
        JobDataMap jobDataMap = new JobDataMap();
        jobDataMap.put("alarmRecord", alarmRecordDO.toString());
        // 已重试次数
        jobDataMap.put("refire", "0");
        jobDataMap.put("endTime", recoverTime);
        jobDataMap.put("endInfo", JSONObject.toJSONString(paramMap));
        jobDataMap.put("defineId", alarmRecordDO.getDefinitionId());
        // 恢复
        jobDataMap.put("state", "2");
        ExpireAlarmMessage em = new ExpireAlarmMessage();
        em.setType("1");
        em.setStartTime(DateUtil.offsetMinute(new Date(), 3).toJdkDate());
        em.setJobDataMap(jobDataMap);
        em.setJobName(alarmRecordDO.getAlarmId());
        em.setJobGroupName("resume");
        ExpireAlarmQueue.getExpireAlarmMessageQueue().produce(em);
    }

    /**
     * @description: 发送报警恢复消息
     * @param: projectId 项目id
     * @param: recoverTime 恢复时间
     * @param: paramMap 采集值
     * @param: alarmId 报警id
     * @return: com.persagy.entity.NettyMessage<com.persagy.entity.AlarmRecord>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/7 10:03 上午
     * @version: V1.0
     */
    private NettyMessage<AlarmRecord> sendAlarmRecoverMessage(
            String projectId, String recoverTime, HashMap<String, Object> paramMap, String alarmId)
            throws InterruptedException {
        //报警恢复参数
        AlarmRecord alarmRecoverRecord = AlarmRecord.builder()
                .state(2)
                .groupCode(groupCode)
                .projectId(projectId)
                .endTime(DateUtils.parseDate(recoverTime))
                .endInfo(JSONObject.toJSONString(paramMap))
                .build();
        alarmRecoverRecord.setId(alarmId);
        NettyMessage<AlarmRecord> nettyMessage = new NettyMessage<>(6, projectId);
        nettyMessage.setContent(Collections.singletonList(alarmRecoverRecord));
        groupNettyClient.sendMessage(nettyMessage.toString());
        return nettyMessage;
    }

    /**
     * @description: 取消报警的过期任务
     * @param: alarmId
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/2/7 9:45 上午
     * @version: V1.0
     */
    private void cancelRelatedExpireJob(String alarmId) throws InterruptedException {
        ExpireAlarmMessage em = new ExpireAlarmMessage();
        em.setType("2");
        em.setJobName(alarmId);
        em.setJobGroupName("expire");
        ExpireAlarmQueue.getExpireAlarmMessageQueue().produce(em);
    }

    /**
     * @description: 当报警恢复时边缘端删除报警信息
     * @param: defineId 报警定义id
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/13 下午5:11
     * @version: V1.0
     */
    private void deleteZktAlarmRecordWhenAlarmSuspend(String defineId) {
        if (alarmRecordRepository.existsById(defineId)) {
            alarmRecordRepository.deleteById(defineId);
        }
    }

    /**
     * @description: 当报警恢复时拼装边缘端报警信息
     * @param: defineId 报警定义id
     * @param: dataTime
     * @param: paramMap
     * @return: com.persagy.entity.ZktAlarmRecordDO
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/13 下午5:11
     * @version: V1.0
     */
    private ZktAlarmRecordDO getZktAlarmRecordDOWhenAlarmSuspend(String defineId, String dataTime, HashMap<String, Object> paramMap) {
        if (!alarmRecordRepository.existsById(defineId)) {
            log.error("根据报警定义id[{}]未获取到边缘端报警信息", defineId);
            throw new RuntimeException("根据报警定义id" + defineId + "未获取到边缘端报警信息");
        }
        Optional<ZktAlarmRecordDO> recordOptional = alarmRecordRepository.findById(defineId);
        ZktAlarmRecordDO alarmRecordDO = recordOptional.get();
        alarmRecordDO.setState("2");
        alarmRecordDO.setEndTime(DateUtils.parseDate(dataTime));
        alarmRecordDO.setEndInfo(JSONObject.toJSONString(paramMap));
        return alarmRecordDO;
    }

    /**
     * @description: 发送报警仍在继续的消息
     * @param: alarmId 报警记录id
     * @param: lastTime 报警持续时间（单位：s）
     * @param: paramMap 当前采集值
     * @param: projectId 项目id
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/17 4:40 下午
     * @version: V1.0
     */
    private void initAlarmContinueMsg(String alarmId, long lastTime, HashMap<String, Object> paramMap, String projectId)
            throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode objectNode = objectMapper.createObjectNode();
        objectNode.put("id", alarmId);
        objectNode.put("lastTime", lastTime);
        objectNode.set("iotData", objectMapper.readTree(objectMapper.writeValueAsString(paramMap)));
        objectNode.put("groupCode", groupCode);
        alarmLastTimeCache.setAlarmLastMsg(alarmId, projectId, objectNode);
    }

    /**
     * @param alarmDefine 报警定义
     * @param dataTime    :IOT数据采集时间
     * @param paramMap    报警触发值
     * @description: 处理当前值报警的情况
     * @exception:
     * @author: LuoGuangyi
     * @company: Persagy Technology Co.,Ltd
     * @return: void
     * @since: 2020/10/20 22:31
     * @version: V1.0
     */
    private void handleAlarmTriggerData(
            AlarmDefine alarmDefine, String dataTime,
            HashMap<String, Object> paramMap) throws Exception {
        // 根据defineId获取报警定义状态
        String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
        AlarmDefineState alarmDefineState = getAlarmDefineStateFromCacheOrDb(defineId);

        Condition condition = alarmDefine.getCondition();
        // 判断报警是否在有效期内
        boolean inEffectTime = ConditionUtils.inEffectiveTime(condition, dataTime);

        if (inEffectTime) {
            // 已经产生报警
            if (AlarmDefineState.State.NOT_DEAL.getType().equals(alarmDefineState.getState())) {
                // 获取报警持续时间
                String alarmStartTime = alarmDefineState.getAlarmStartTime();
                long lastTime = DateUtils.betweenTwoTimeSecond(alarmStartTime, dataTime);
                String alarmId = alarmRecordRepository.findById(defineId).orElse(new ZktAlarmRecordDO()).getAlarmId();
                if (StringUtils.isNotBlank(alarmId)) {
                    alarmLastTimeCache.put(alarmId, Integer.valueOf(String.valueOf(lastTime)));
                    log.info("开始拼装报警持续消息");
                    initAlarmContinueMsg(alarmId, lastTime, paramMap, alarmDefine.getProjectId());
                }
            }
            //之前是是正常状态
            if (AlarmDefineState.State.NORMAL.getType().equals(alarmDefineState.getState())) {
                // 报警持续时间
                long alarmLastTime = 0;
                if (StringUtil.isNotEmpty(alarmDefineState.getAlarmStartTime())) {
                    alarmLastTime = DateUtils.betweenTwoTimeSecond(alarmDefineState.getAlarmStartTime(), dataTime);
                } else {
                    //设置开始报警时间
                    alarmDefineState.setAlarmStartTime(dataTime);
                }

                // 获取报警过期时间
                LocalDateTime expireDateTime = getExpireLocalDateTime(dataTime, condition);
                log.info("报警持续时间：[{}]", alarmLastTime);
                log.info("设定的持续时间：[{}]", condition.getTriggerUphold());
                if (alarmLastTime >= condition.getTriggerUphold()) {
                    String alarmId = StringUtil.getUUID();
                    log.info("报警持续时间大于设定的持续时间，产生一条报警: [{}]", alarmId);
                    // 有新的报警产生，alarmLastTimeMap中报警标记为可删除
                    alarmLastTimeCache.setDeleteAvailable(defineId);
                    alarmLastTimeCache.updateAlarmId(defineId, alarmId);
                    // 边缘端保存报警信息
                    saveZktAlarmRecordWhenAlarmStart(alarmId, alarmDefine, dataTime);
                    // 报警定义状态更新为未处理
                    alarmDefineState.setState(AlarmDefineState.State.NOT_DEAL.getType());
                    // 发送创建报警记录的消息
                    sendCreateAlarmRecordMessage(alarmId, alarmDefine, dataTime, paramMap,
                            DateUtils.localDateTime2Date(expireDateTime));

                    //有过期时间，生成报警过期消息
                    if (expireDateTime != null) {
                        //过期时间
                        log.info("产生一条定时过期报警消息, 过期时间为：{}", expireDateTime);
                        // 创建一条过期任务
                        createExpireJob(alarmId, alarmDefine, alarmDefineState, expireDateTime);
                    }
                }
            }
        } else {
            // 其他条件全部改成默认值（不报警，不过期，报警开始时间和结束时间为空）
            log.info("[{}]不在生效时间，重置报警状态", dataTime);
            log.info("生效时间为：[{}]", condition.getEffectTime());
            alarmDefineState.reset();
        }
        alarmInfoCache.setAlarmState(defineId, alarmDefineState);
    }

    /**
     * @description: 获取报警过期时间
     * @param: triggerTime 报警触发时间
     * @param: condition 报警条件
     * @return: java.time.LocalDateTime
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午6:16
     * @version: V1.0
     */
    private LocalDateTime getExpireLocalDateTime(String triggerTime, Condition condition) {
        // 判断过期设置是否生效
        boolean expireSetEffective = ConditionUtils.expireSetEffective(condition);

        if (!expireSetEffective) {
            return null;
        }

        //过期时间
        String expireTime = ConditionUtils.getEffectEndTime(condition);
        LocalTime localTime = LocalTime.parse(expireTime, DateTimeFormatter.ofPattern(DateUtils.sdfTimeNotDate));
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DateUtils.sdfDay);
        LocalDate triggerDate = LocalDate.parse(DateUtils.getDate(triggerTime), formatter);
        return LocalDateTime.of(triggerDate, localTime);
    }

    /**
     * @description: 创建过期任务
     * @param: alarmId 报警id
     * @param: alarmDefine 报警定义
     * @param: alarmDefineState 报警定义状态对象
     * @param: expireDateTime 过期时间
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/13 下午3:39
     * @version: V1.0
     */
    private void createExpireJob(String alarmId, AlarmDefine alarmDefine, AlarmDefineState alarmDefineState, LocalDateTime expireDateTime) throws InterruptedException {
        String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
        ZktAlarmRecordDO alarmRecordDO = ZktAlarmRecordDO.builder()
                .alarmTime(alarmDefineState.getAlarmStartTime())
                .effectEndTime(DateUtils.format(expireDateTime))
                .definitionId(defineId)
                .itemCode(alarmDefine.getItemCode())
                .name(alarmDefine.getName())
                .objId(alarmDefine.getObjId())
                //要变成已过期
                .state("3")
                .alarmId(alarmId)
                .build();
        JobDataMap jobDataMap = new JobDataMap();
        jobDataMap.put("alarmRecord", alarmRecordDO.toString());
        jobDataMap.put("refire", "0");
        jobDataMap.put("expireTime", DateUtils.format(expireDateTime));
        jobDataMap.put("defineId", defineId);
        //过期
        jobDataMap.put("state", "3");
        ExpireAlarmMessage em = new ExpireAlarmMessage();
        //过期消息
        em.setType("1");
        em.setStartTime(DateUtils.localDateTime2Date(expireDateTime));
        em.setJobDataMap(jobDataMap);
        em.setJobName(alarmId);
        em.setJobGroupName("expire");
        ExpireAlarmQueue.getExpireAlarmMessageQueue().produce(em);
    }

    /**
     * @description: 当报警开始时边缘端保存报警信息
     * @param: alarmId 报警记录id
     * @param: alarmDefine 报警定义
     * @return: com.persagy.entity.ZktAlarmRecordDO
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午5:55
     * @version: V1.0
     */
    private ZktAlarmRecordDO saveZktAlarmRecordWhenAlarmStart(String alarmId, AlarmDefine alarmDefine, String alarmTime) {
        String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
        synchronized (defineId.intern()) {
            ZktAlarmRecordDO zktAlarmRecordDO = initZktAlarmRecordDO(alarmDefine);
            zktAlarmRecordDO.setAlarmId(alarmId);
            zktAlarmRecordDO.setState("1");
            zktAlarmRecordDO.setAlarmTime(alarmTime);
            alarmRecordRepository.save(zktAlarmRecordDO);
            return zktAlarmRecordDO;
        }
    }

    /**
     * @description: 初始化边缘端报警记录对象
     * @param: alarmDefine
     * @return: com.persagy.entity.ZktAlarmRecordDO
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/13 下午5:07
     * @version: V1.0
     */
    private ZktAlarmRecordDO initZktAlarmRecordDO(AlarmDefine alarmDefine) {
        String defineId = AlarmInfoCache.getAlarmDefineId(alarmDefine);
        ZktAlarmRecordDO zktAlarmRecordDO = alarmRecordRepository.findById(defineId).orElse(new ZktAlarmRecordDO());
        zktAlarmRecordDO.setDefinitionId(defineId);
        zktAlarmRecordDO.setObjId(alarmDefine.getObjId());
        zktAlarmRecordDO.setItemCode(alarmDefine.getItemCode());
        zktAlarmRecordDO.setProjectId(alarmDefine.getProjectId());
        return zktAlarmRecordDO;
    }


    /**
     * @description: 发送创建报警记录消息
     * @param: alarmId 报警id
     * @param: alarmDefine 报警定义
     * @param: triggerTime 触发时间
     * @param: triggerInfo 触发值
     * @param: expireDate 过期时间
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/8 下午5:38
     * @version: V1.0
     */
    private void sendCreateAlarmRecordMessage(
            String alarmId,
            AlarmDefine alarmDefine, String triggerTime,
            HashMap<String, Object> triggerInfo, Date expireDate) throws InterruptedException {
        Condition condition = alarmDefine.getCondition();
        AlarmRecord alarmRecord = AlarmRecord.builder()
                .id(alarmId)
                .category(alarmDefine.getCategory())
                .concern(alarmDefine.getConcern())
                .level(alarmDefine.getLevel())
                .projectId(alarmDefine.getProjectId())
                .state(1)
                .triggerTime(DateUtils.parseDate(triggerTime))
                .remark(alarmDefine.getRemark())
                .triggerInfo(JSONObject.toJSONString(triggerInfo))
                .condition(condition.toString())
                .effectEndTime(expireDate)
                .groupCode(groupCode)
                .itemCode(alarmDefine.getItemCode())
                .objId(alarmDefine.getObjId())
                .classCode(alarmDefine.getClassCode())
                .createUser(systemId)
                .treatState(1)
                .build();
        NettyMessage<AlarmRecord> nettyMessage = new NettyMessage<>(5, alarmDefine.getProjectId());
        nettyMessage.setContent(Collections.singletonList(alarmRecord));
        //推送一条报警记录给远端
        groupNettyClient.sendMessage(nettyMessage.toString());
    }
}
